/*
 * This file is part of CutAndPlay.
 *
 * Copyright (C) 2009 Stefan Westerfeld
 * Copyright (C) 2009 Treuwert Computer GmbH
 *
 * CutAndPlay is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package cutandplay;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.event.KeyEvent;
import java.awt.geom.Rectangle2D;
import java.awt.Stroke;
import java.util.ListIterator;
import java.util.Vector;
import javax.xml.stream.XMLStreamConstants;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;
import javax.xml.stream.XMLStreamWriter;

/**
 *
 * @author stefan
 */
public class Box extends SlideElement {
    Rectangle space;
    Dimension virtual_screen;
    Slide slide;
    Color bgColor;

    Point clickPosition;        // for dragging
    Point spacePosition;        // for dragging
    Dimension spaceDimension;   // for dragging
    DragHandle clickHandle;     // for dragging
    Vector dragHandles;
    Box (Slide slide)
    {
        this.slide = slide;
        space = new Rectangle (50, 50, 500, 500);
        virtual_screen = new Dimension (3000 * slide.presentation.aspect_x / slide.presentation.aspect_y, 3000);
        bgColor = Color.GRAY;
        dragHandles = new Vector();
        double factors[] = {   0, 0, 0, 0.5,   0, 1,
                             0.5, 0,         0.5, 1,
                               1, 0, 1, 0.5,   1, 1 };
        for (int i = 0; i < factors.length; i += 2)
            dragHandles.add (new DragHandle (factors[i], factors[i+1]));
    }
    @Override
    boolean at(Point p, Rectangle r) {
        double vx = (double) (p.x - r.x) / r.width * virtual_screen.width;
        double vy = (double) (p.y - r.y) / r.height * virtual_screen.height;
        if (space.contains (new Point ((int)vx, (int)vy)))
            return true;
        if (findHandle (p, r) != null && selected)
            return true;
        return false;
    }
    DragHandle findHandle (Point p, Rectangle r)
    {
        double xpos = (double) (space.x * r.width) / virtual_screen.width;
        double ypos = (double) (space.y * r.height) / virtual_screen.height;
        double width = (double) (space.width * r.width) / virtual_screen.width;
        double height = (double) (space.height * r.height) / virtual_screen.height;
        ListIterator iter = dragHandles.listIterator();
        while (iter.hasNext())
        {
            DragHandle dh  = (DragHandle) iter.next();
            int x = r.x + (int)(xpos + dh.delta_x * width) - 5;
            int y = r.y + (int)(ypos + dh.delta_y * height) - 5;
            if (new Rectangle (x, y, 10, 10).contains (p))
                return dh;
        }
        return null;
    }
    @Override
    void mousePressed(Point p, Rectangle r) {
        clickHandle = findHandle (p, r);
        clickPosition = p;
        spacePosition = new Point (space.x, space.y);
        spaceDimension = new Dimension (space.width, space.height);
    }

    @Override
    void mouseDragged(Point p, Rectangle r) {
        double vx = (double) (p.x - clickPosition.x) / r.width * virtual_screen.width;
        double vy = (double) (p.y - clickPosition.y) / r.height * virtual_screen.height;

        if (clickHandle != null) /* RESIZE */
        {
            if (clickHandle.delta_x == 0.0)
            {
                if (vx < -spacePosition.x)
                    vx = -spacePosition.x;
                if (vx > spaceDimension.width - 50)
                    vx = spaceDimension.width - 50;
                space.x = spacePosition.x + (int) vx;
                space.width = spaceDimension.width - (int) vx;
            }
            if (clickHandle.delta_y == 0.0)
            {
                if (vy < -spacePosition.y)
                    vy = -spacePosition.y;
                if (vy > spaceDimension.height - 50)
                    vy = spaceDimension.height - 50;
                space.y = spacePosition.y + (int) vy;
                space.height = spaceDimension.height - (int) vy;
            }
            if (clickHandle.delta_x == 1.0)
            {
                space.width = spaceDimension.width + (int) vx;
                if (space.width < 50)
                    space.width = 50;
            }
            if (clickHandle.delta_y == 1.0)
            {
                space.height = spaceDimension.height + (int) vy;
                if (space.height < 50)
                    space.height = 50;
            }
            int x_outside = space.x + space.width - virtual_screen.width;
            if (x_outside > 0)
                space.width -= x_outside;
            int y_outside = space.y + space.height - virtual_screen.height;
            if (y_outside > 0)
                space.height -= y_outside;
        }
        else /* MOVE */
        {
            space.x = spacePosition.x + (int)vx;
            space.y = spacePosition.y + (int)vy;
            if (space.x < 0)
                space.x = 0;
            if (space.y < 0)
                space.y = 0;
            int x_outside = space.x + space.width - virtual_screen.width;
            if (x_outside > 0)
                space.x -= x_outside;
            int y_outside = space.y + space.height - virtual_screen.height;
            if (y_outside > 0)
                space.y -= y_outside;
        }
        slide.view.redraw();
    }

    @Override
    void mouseReleased(Point p, Rectangle r) {
    }

    @Override
    void keyTyped(KeyEvent e) {
    }

    @Override
    void draw(Graphics2D g2, Rectangle r, boolean draw_selection) {
        double xpos = (double) (space.x * r.width) / virtual_screen.width;
        double ypos = (double) (space.y * r.height) / virtual_screen.height;
        double width = (double) (space.width * r.width) / virtual_screen.width;
        double height = (double) (space.height * r.height) / virtual_screen.height;
        Shape shape = new Rectangle2D.Float (r.x + (int)xpos, r.y + (int)ypos, (int)width, (int)height);
        g2.setColor (bgColor);
        g2.fill(shape);
        if (selected && draw_selection)
        {
            float dash1[] = { 10.0f };
            BasicStroke dashed = new BasicStroke(1.0f,
                         BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER, 10.0f, dash1, 0.0f);
            g2.setColor(Color.BLACK);
            g2.draw(shape);
            g2.setColor (Color.YELLOW);
            Stroke orig_stroke = g2.getStroke();
            g2.setStroke (dashed);
            g2.draw(shape);
            g2.setStroke (orig_stroke);
            ListIterator iter = dragHandles.listIterator();
            while (iter.hasNext())
            {
                DragHandle dh  = (DragHandle) iter.next();
                drawHandle (g2, r.x + (int)(xpos + dh.delta_x * width) - 5, r.y + (int) (ypos + dh.delta_y * height) - 5);
            }
        }
    }
    void drawHandle (Graphics2D g2, int x, int y)
    {
       g2.setColor(Color.WHITE);
       g2.fillRect(x, y, 10, 10);
       g2.setColor(Color.BLACK);
       g2.drawRect(x, y, 9, 9);
       g2.drawLine(x + 2, y + 4, x + 4, y + 2);
       g2.drawLine(x + 2, y + 7, x + 7, y + 2);
       g2.drawLine(x + 5, y + 7, x + 7, y + 5);
    }

    @Override
    void showToolPanel(ToolPanelProvider provider) {
        provider.showBoxPanel (this);
    }
    Color getBgColor() {
        return bgColor;
    }
    void setBgColor (Color color) {
        bgColor = color;
        slide.view.redraw();
    }

    @Override
    int layer() {
        return 1;
    }

    @Override
    void save (XMLStreamWriter xtw) throws XMLStreamException {
        xtw.writeStartElement("box");
        xtw.writeAttribute("x", Integer.toString (space.x));
        xtw.writeAttribute("y", Integer.toString (space.y));
        xtw.writeAttribute("width", Integer.toString (space.width));
        xtw.writeAttribute("height", Integer.toString (space.height));
        writeAttributeColor(xtw, "bg_color", bgColor);
        xtw.writeEndElement();
    }
    
    @Override
    void load (XMLStreamReader r) throws XMLStreamException
    {
        for (int i = 0; i < r.getAttributeCount(); i++)
        {
            String attr_name = r.getAttributeLocalName (i);
            String attr_value = r.getAttributeValue (i);
            if (attr_name.equals ("x"))
                space.x = Integer.parseInt (attr_value);
            if (attr_name.equals ("y"))
                space.y = Integer.parseInt (attr_value);
            if (attr_name.equals ("width"))
                space.width = Integer.parseInt (attr_value);
            if (attr_name.equals ("height"))
                space.height = Integer.parseInt (attr_value);
            if (attr_name.equals ("bg_color"))
                bgColor = parseAttributeColor (attr_value);
        }

        while (r.hasNext())
        {
            int event = r.next();
            if (event == XMLStreamConstants.END_ELEMENT)
                if (r.getLocalName().equals ("box"))
                    return;
        }
    }
}
