/*
 * This file is part of CutAndPlay.
 *
 * Copyright (C) 2009 Stefan Westerfeld
 * Copyright (C) 2009 Treuwert Computer GmbH
 *
 * CutAndPlay is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package cutandplay;

import java.awt.event.ComponentEvent;
import java.awt.event.KeyEvent;
import javax.swing.*;
import java.awt.*;
import java.awt.event.ComponentListener;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.util.ListIterator;

/**
 *
 * @author stefan
 */
public class SlideView extends JPanel {
     public Slide slide;
     public SlideElement focusElement;
     public InnerListener listener;
     public ToolPanelProvider toolPanelProvider;
     public SlideView() {
        listener = new InnerListener();
        this.addComponentListener (listener);
        this.addMouseListener (listener);
        this.addMouseMotionListener (listener);
        this.addMouseWheelListener (listener);
        this.addKeyListener (listener);
     }
     public void setSlide (Slide slide)
     {
         this.slide = slide;
	 if (slide != null)
	 {
             slide.view = this;
	     redraw();
	 }
     }
     public void redraw()
     {
         repaint();
     }
    Rectangle drawingRectangle()
    {
	int aspect_x = 4;
	int aspect_y = 3;
	if (slide != null)
	{
	    aspect_x = slide.presentation.aspect_x;
	    aspect_y = slide.presentation.aspect_y;
	}

	int x = 0, y = 0;
	int w = getWidth();
	int h = getHeight();
	if  (w / aspect_x > h / aspect_y)
	{
	    int oldw = w;
	    w = h * aspect_x / aspect_y;
	    x = (oldw - w) / 2;
	}
	else if (h / aspect_y > w / aspect_x)
	{
	    int oldh = h;
	    h = w * aspect_y / aspect_x;
	    y = (oldh - h) / 2;
	}
	return new Rectangle(x, y, w, h);
    }
    protected void paintComponent(Graphics g)
    {
	super.paintComponent(g);
	if (slide != null)
	{
	    Rectangle drect = drawingRectangle();
	    Graphics2D g2 = (Graphics2D)g;
	    g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
		    RenderingHints.VALUE_ANTIALIAS_ON);
	    slide.draw (g2, drect, true);
	}
    }
    public void setFocus(SlideElement newFocus)
     {
	 if (focusElement != newFocus)
	 {
	     requestFocusInWindow();
	     focusElement = newFocus;
	     ListIterator iter = slide.elements.listIterator();
	     while (iter.hasNext()) {
		 SlideElement element = (SlideElement) iter.next();
		 element.setSelected (focusElement == element);
	     }
	     if (focusElement != null)
		 focusElement.showToolPanel(toolPanelProvider);
	     else
		 toolPanelProvider.hidePanels();
	     redraw();
	 }
     }
     private class InnerListener extends MouseAdapter implements ComponentListener, MouseMotionListener, MouseWheelListener, KeyListener {
        SlideElement mouseElement;
        @Override
        public void mousePressed(MouseEvent e) {
            SlideElement newFocusElement = null;
            Point cursorPosition = e.getPoint();
            e.getComponent().requestFocusInWindow();
            ListIterator iter = slide.elements.listIterator();
            while (iter.hasNext()) {
                SlideElement element = (SlideElement) iter.next();
                if (element.at (cursorPosition, drawingRectangle()))
                {
                    mouseElement = element;
                    newFocusElement = element;
                    element.mousePressed (cursorPosition, drawingRectangle());
                }
            }
	    setFocus (newFocusElement);
        }

        @Override
        public void mouseReleased(MouseEvent e) {
            Point cursorPosition = e.getPoint();
            if (mouseElement != null)
            {
                mouseElement.mouseReleased (cursorPosition, drawingRectangle());
                mouseElement = null;
            }
        }

        public void mouseDragged(MouseEvent e) {
            Point cursorPosition = e.getPoint();
            if (mouseElement != null)
                mouseElement.mouseDragged (cursorPosition, drawingRectangle());
        }

        public void mouseWheelMoved(MouseWheelEvent e) {
        }

        public void componentMoved(ComponentEvent e) {}
        public void componentShown(ComponentEvent e) {}
        public void componentHidden(ComponentEvent e) {}
        public void mouseMoved(MouseEvent e) {}
        public void componentResized(ComponentEvent arg0) {}

        public void keyTyped(KeyEvent e) {
            if (focusElement != null)
                focusElement.keyTyped (e);
	    if (e.getKeyChar() == KeyEvent.VK_DELETE)
		slide.deleteSelected();
        }

        public void keyPressed(KeyEvent arg0) {
        }

        public void keyReleased(KeyEvent arg0) {            
        }
     }

}
